@@ -18,7 +18,7 @@
 // |          Tomas V.V.Cox <cox@idecnet.com>                             |
 // +----------------------------------------------------------------------+
 //
-// $Id: PEAR.php,v 1.32.2.2 2002/04/09 19:04:07 ssb Exp $
+// $Id: PEAR.php,v 1.50 2002/10/08 13:17:01 pajoye Exp $
 //
 
 define('PEAR_ERROR_RETURN',   1);
@@ -26,6 +26,8 @@ define('PEAR_ERROR_PRINT',    2);
 define('PEAR_ERROR_TRIGGER',  4);
 define('PEAR_ERROR_DIE',      8);
 define('PEAR_ERROR_CALLBACK', 16);
+define('PEAR_ZE2', (function_exists('version_compare') &&
+                    version_compare(zend_version(), "2-dev", "ge")));
 
 if (substr(PHP_OS, 0, 3) == 'WIN') {
     define('OS_WINDOWS', true);
@@ -39,12 +41,11 @@ if (substr(PHP_OS, 0, 3) == 'WIN') {
 
 $GLOBALS['_PEAR_default_error_mode']     = PEAR_ERROR_RETURN;
 $GLOBALS['_PEAR_default_error_options']  = E_USER_NOTICE;
-$GLOBALS['_PEAR_default_error_callback'] = '';
 $GLOBALS['_PEAR_destructor_object_list'] = array();
+$GLOBALS['_PEAR_shutdown_funcs']         = array();
+$GLOBALS['_PEAR_error_handler_stack']    = array();
 
-//
-// Tests needed: - PEAR inheritance
-//
+ini_set('track_errors', true);
 
 /**
  * Base class for other PEAR classes.  Provides rudimentary
@@ -60,8 +61,12 @@ $GLOBALS['_PEAR_destructor_object_list'] = array();
  * discarded.  If you need to get any debug information from your
  * destructor, use error_log(), syslog() or something similar.
  *
+ * IMPORTANT! To use the emulated destructors you need to create the
+ * objects by reference, ej: $obj =& new PEAR_child;
+ *
  * @since PHP 4.0.2
  * @author Stig Bakken <ssb@fast.no>
+ * @see http://pear.php.net/manual/
  */
 class PEAR
 {
@@ -126,8 +131,8 @@ class PEAR
      * $_PEAR_destructor_object_list for destructor emulation if a
      * destructor object exists.
      *
-     * @param string      (optional) which class to use for error objects,
-     *                    defaults to PEAR_Error.
+     * @param string $error_class  (optional) which class to use for
+     *        error objects, defaults to PEAR_Error.
      * @access public
      * @return void
      */
@@ -172,13 +177,51 @@ class PEAR
         }
     }
 
+    // }}}
+    // {{{ getStaticProperty()
+
+    /**
+    * If you have a class that's mostly/entirely static, and you need static
+    * properties, you can use this method to simulate them. Eg. in your method(s)
+    * do this: $myVar = &PEAR::getStaticProperty('myVar');
+    * You MUST use a reference, or they will not persist!
+    *
+    * @access public
+    * @param  string $class  The calling classname, to prevent clashes
+    * @param  string $var    The variable to retrieve.
+    * @return mixed   A reference to the variable. If not set it will be
+    *                 auto initialised to NULL.
+    */
+    function &getStaticProperty($class, $var)
+    {
+        static $properties;
+        return $properties[$class][$var];
+    }
+
+    // }}}
+    // {{{ registerShutdownFunc()
+
+    /**
+    * Use this function to register a shutdown method for static
+    * classes.
+    *
+    * @access public
+    * @param  mixed $func  The function name (or array of class/method) to call
+    * @param  mixed $args  The arguments to pass to the function
+    * @return void
+    */
+    function registerShutdownFunc($func, $args = array())
+    {
+        $GLOBALS['_PEAR_shutdown_funcs'][] = array($func, $args);
+    }
+
     // }}}
     // {{{ isError()
 
     /**
      * Tell whether a value is a PEAR error.
      *
-     * @param   mixed   the value to test
+     * @param   mixed $data   the value to test
      * @access  public
      * @return  bool    true if parameter is an error
      */
@@ -234,11 +277,9 @@ class PEAR
         if (isset($this)) {
             $setmode     = &$this->_default_error_mode;
             $setoptions  = &$this->_default_error_options;
-            //$setcallback = &$this->_default_error_callback;
         } else {
             $setmode     = &$GLOBALS['_PEAR_default_error_mode'];
             $setoptions  = &$GLOBALS['_PEAR_default_error_options'];
-            //$setcallback = &$GLOBALS['_PEAR_default_error_callback'];
         }
 
         switch ($mode) {
@@ -279,12 +320,14 @@ class PEAR
      * expected errors are in effect until they are popped off the
      * stack with the popExpect() method.
      *
-     * @param mixed    a single error code or an array of error codes
-     *                 to expect
+     * Note that this method can not be called statically
+     *
+     * @param mixed $code a single error code or an array of error codes to expect
      *
      * @return int     the new depth of the "expected errors" stack
+     * @access public
      */
-    function expectError($code = "*")
+    function expectError($code = '*')
     {
         if (is_array($code)) {
             array_push($this->_expected_errors, $code);
@@ -308,6 +351,76 @@ class PEAR
         return array_pop($this->_expected_errors);
     }
 
+    // }}}
+    // {{{ _checkDelExpect()
+
+    /**
+     * This method checks unsets an error code if available
+     *
+     * @param mixed error code
+     * @return bool true if the error code was unset, false otherwise
+     * @access private
+     * @since PHP 4.3.0
+     */
+    function _checkDelExpect($error_code)
+    {
+        $deleted = false;
+
+        foreach ($this->_expected_errors AS $key => $error_array) {
+            if (in_array($error_code, $error_array)) {
+                unset($this->_expected_errors[$key][array_search($error_code, $error_array)]);
+                $deleted = true;
+            }
+
+            // clean up empty arrays
+            if (0 == count($this->_expected_errors[$key])) {
+                unset($this->_expected_errors[$key]);
+            }
+        }
+        return $deleted;
+    }
+
+    // }}}
+    // {{{ delExpect()
+
+    /**
+     * This method deletes all occurences of the specified element from
+     * the expected error codes stack.
+     *
+     * @param  mixed $error_code error code that should be deleted
+     * @return mixed list of error codes that were deleted or error
+     * @access public
+     * @since PHP 4.3.0
+     */
+    function delExpect($error_code)
+    {
+        $deleted = false;
+
+        if ((is_array($error_code) && (0 != count($error_code)))) {
+            // $error_code is a non-empty array here;
+            // we walk through it trying to unset all
+            // values
+            foreach($error_code AS $key => $error) {
+                if ($this->_checkDelExpect($error)) {
+                    $deleted =  true;
+                } else {
+                    $deleted = false;
+                }
+            }
+            return $deleted ? true : PEAR::raiseError("The expected error you submitted does not exist"); // IMPROVE ME
+        } elseif (!empty($error_code)) {
+            // $error_code comes alone, trying to unset it
+            if ($this->_checkDelExpect($error_code)) {
+                return true;
+            } else {
+                return PEAR::raiseError("The expected error you submitted does not exist"); // IMPROVE ME
+            }
+        } else {
+            // $error_code is empty
+            return PEAR::raiseError("The expected error you submitted is empty"); // IMPROVE ME
+        }
+    }
+
     // }}}
     // {{{ raiseError()
 
@@ -317,16 +430,16 @@ class PEAR
      * handling applied.  If the $mode and $options parameters are not
      * specified, the object's defaults are used.
      *
-     * @param $message  a text error message or a PEAR error object
+     * @param mixed $message a text error message or a PEAR error object
      *
-     * @param $code     a numeric error code (it is up to your class
+     * @param int $code      a numeric error code (it is up to your class
      *                  to define these if you want to use codes)
      *
-     * @param $mode     One of PEAR_ERROR_RETURN, PEAR_ERROR_PRINT,
+     * @param int $mode      One of PEAR_ERROR_RETURN, PEAR_ERROR_PRINT,
      *                  PEAR_ERROR_TRIGGER, PEAR_ERROR_DIE or
      *                  PEAR_ERROR_CALLBACK.
      *
-     * @param $options  If $mode is PEAR_ERROR_TRIGGER, this parameter
+     * @param mixed $options If $mode is PEAR_ERROR_TRIGGER, this parameter
      *                  specifies the PHP-internal error level (one of
      *                  E_USER_NOTICE, E_USER_WARNING or E_USER_ERROR).
      *                  If $mode is PEAR_ERROR_CALLBACK, this
@@ -334,13 +447,13 @@ class PEAR
      *                  method.  In other error modes this parameter
      *                  is ignored.
      *
-     * @param $userinfo If you need to pass along for example debug
+     * @param string $userinfo If you need to pass along for example debug
      *                  information, this parameter is meant for that.
      *
-     * @param $error_class The returned error object will be instantiated
-     *                  from this class, if specified.
+     * @param string $error_class The returned error object will be
+     *                  instantiated from this class, if specified.
      *
-     * @param $skipmsg  If true, raiseError will only pass error codes,
+     * @param bool $skipmsg If true, raiseError will only pass error codes,
      *                  the error message parameter will be dropped.
      *
      * @access public
@@ -371,47 +484,19 @@ class PEAR
                 $mode = PEAR_ERROR_RETURN;
             }
         }
-
+        // No mode given, try global ones
         if ($mode === null) {
+            // Class error handler
             if (isset($this) && isset($this->_default_error_mode)) {
-                $mode = $this->_default_error_mode;
-            } else {
-                $mode = $GLOBALS['_PEAR_default_error_mode'];
-            }
-        }
-
-        if ($mode == PEAR_ERROR_TRIGGER && $options === null) {
-            if (isset($this)) {
-                if (isset($this->_default_error_options)) {
-                    $options = $this->_default_error_options;
-                }
-            } else {
+                $mode    = $this->_default_error_mode;
+                $options = $this->_default_error_options;
+            // Global error handler
+            } elseif (isset($GLOBALS['_PEAR_default_error_mode'])) {
+                $mode    = $GLOBALS['_PEAR_default_error_mode'];
                 $options = $GLOBALS['_PEAR_default_error_options'];
             }
         }
 
-        if ($mode == PEAR_ERROR_CALLBACK) {
-            if (!is_string($options) &&
-                !(is_array($options) && sizeof($options) == 2 &&
-                  is_object($options[0]) && is_string($options[1])))
-            {
-                if (isset($this) && isset($this->_default_error_options)) {
-                    $options = $this->_default_error_options;
-                } else {
-                    $options = $GLOBALS['_PEAR_default_error_options'];
-                }
-            }
-        } else {
-            if ($options === null) {
-                if (isset($this)) {
-                    if (isset($this->_default_error_options)) {
-                        $options = $this->_default_error_options;
-                    }
-                } else {
-                    $options = $GLOBALS['_PEAR_default_error_options'];
-                }
-            }
-        }
         if ($error_class !== null) {
             $ec = $error_class;
         } elseif (isset($this) && isset($this->_error_class)) {
@@ -426,39 +511,53 @@ class PEAR
         }
     }
 
+    // }}}
+    // {{{ throwError()
+
+    /**
+     * Simpler form of raiseError with fewer options.  In most cases
+     * message, code and userinfo are enough.
+     *
+     * @param string $message
+     *
+     */
+    function &throwError($message = null,
+                         $code = null,
+                         $userinfo = null)
+    {
+        if (isset($this)) {
+            return $this->raiseError($message, $code, null, null, $userinfo);
+        } else {
+            return PEAR::raiseError($message, $code, null, null, $userinfo);
+        }
+    }
+
     // }}}
     // {{{ pushErrorHandling()
 
     /**
-    * Push a new error handler on top of the error handler options stack. With this
-    * you can easily override the actual error handler for some code and restore
-    * it later with popErrorHandling.
-    *
-    * @param $mode mixed (same as setErrorHandling)
-    * @param $options mixed (same as setErrorHandling)
-    *
-    * @return bool Always true
-    *
-    * @see PEAR::setErrorHandling
-    */
+     * Push a new error handler on top of the error handler options stack. With this
+     * you can easily override the actual error handler for some code and restore
+     * it later with popErrorHandling.
+     *
+     * @param mixed $mode (same as setErrorHandling)
+     * @param mixed $options (same as setErrorHandling)
+     *
+     * @return bool Always true
+     *
+     * @see PEAR::setErrorHandling
+     */
     function pushErrorHandling($mode, $options = null)
     {
         $stack = &$GLOBALS['_PEAR_error_handler_stack'];
-        if (!is_array($stack)) {
-            if (isset($this)) {
-                $def_mode = &$this->_default_error_mode;
-                $def_options = &$this->_default_error_options;
-                // XXX Used anywhere?
-                //$def_callback = &$this->_default_error_callback;
-            } else {
-                $def_mode = &$GLOBALS['_PEAR_default_error_mode'];
-                $def_options = &$GLOBALS['_PEAR_default_error_options'];
-                // XXX Used anywhere?
-                //$def_callback = &$GLOBALS['_PEAR_default_error_callback'];
-            }
-            $stack = array();
-            $stack[] = array($def_mode, $def_options);
+        if (isset($this)) {
+            $def_mode    = &$this->_default_error_mode;
+            $def_options = &$this->_default_error_options;
+        } else {
+            $def_mode    = &$GLOBALS['_PEAR_default_error_mode'];
+            $def_options = &$GLOBALS['_PEAR_default_error_options'];
         }
+        $stack[] = array($def_mode, $def_options);
 
         if (isset($this)) {
             $this->setErrorHandling($mode, $options);
@@ -484,6 +583,7 @@ class PEAR
         $stack = &$GLOBALS['_PEAR_error_handler_stack'];
         array_pop($stack);
         list($mode, $options) = $stack[sizeof($stack) - 1];
+        array_pop($stack);
         if (isset($this)) {
             $this->setErrorHandling($mode, $options);
         } else {
@@ -492,6 +592,35 @@ class PEAR
         return true;
     }
 
+    // }}}
+    // {{{ loadExtension()
+
+    /**
+    * OS independant PHP extension load. Remember to take care
+    * on the correct extension name for case sensitive OSes.
+    *
+    * @param string $ext The extension name
+    * @return bool Success or not on the dl() call
+    */
+    function loadExtension($ext)
+    {
+        if (!extension_loaded($ext)) {
+            if (OS_WINDOWS) {
+                $suffix = '.dll';
+            } elseif (PHP_OS == 'HP-UX') {
+                $suffix = '.sl';
+            } elseif (PHP_OS == 'AIX') {
+                $suffix = '.a';
+            } elseif (PHP_OS == 'OSX') {
+                $suffix = '.bundle';
+            } else {
+                $suffix = '.so';
+            }
+            return @dl('php_'.$ext.$suffix) || @dl($ext.$suffix);
+        }
+        return true;
+    }
+
     // }}}
 }
 
@@ -520,6 +649,13 @@ function _PEAR_call_destructors()
         // not called more than once.
         $_PEAR_destructor_object_list = array();
     }
+
+    // Now call the shutdown functions
+    if (is_array($GLOBALS['_PEAR_shutdown_funcs']) AND !empty($GLOBALS['_PEAR_shutdown_funcs'])) {
+        foreach ($GLOBALS['_PEAR_shutdown_funcs'] as $value) {
+            call_user_func_array($value[0], $value[1]);
+        }
+    }
 }
 
 // }}}
@@ -546,18 +682,20 @@ class PEAR_Error
     /**
      * PEAR_Error constructor
      *
-     * @param $message error message
+     * @param string $message  message
      *
-     * @param $code (optional) error code
+     * @param int $code     (optional) error code
      *
-     * @param $mode (optional) error mode, one of: PEAR_ERROR_RETURN,
+     * @param int $mode     (optional) error mode, one of: PEAR_ERROR_RETURN,
      * PEAR_ERROR_PRINT, PEAR_ERROR_DIE, PEAR_ERROR_TRIGGER or
      * PEAR_ERROR_CALLBACK
      *
-     * @param $level (optional) error level, _OR_ in the case of
+     * @param mixed $options   (optional) error level, _OR_ in the case of
      * PEAR_ERROR_CALLBACK, the callback function or object/method
      * tuple.
      *
+     * @param string $userinfo (optional) additional user/debug info
+     *
      * @access public
      *
      */
@@ -612,8 +750,7 @@ class PEAR_Error
                       is_object($this->callback[0]) &&
                       is_string($this->callback[1]) &&
                       strlen($this->callback[1])) {
-                      @call_user_method($this->callback[1], $this->callback[0],
-                                 $this);
+                      @call_user_func($this->callback, $this);
             }
         }
     }
@@ -654,7 +791,7 @@ class PEAR_Error
      * @return  string  full error message
      * @access public
      */
-    function getMessage ()
+    function getMessage()
     {
         return ($this->error_message_prefix . $this->message);
     }
@@ -683,7 +820,7 @@ class PEAR_Error
      * @return string error/exception name (type)
      * @access public
      */
-    function getType ()
+    function getType()
     {
         return get_class($this);
     }
@@ -697,7 +834,7 @@ class PEAR_Error
      * @return string user-supplied information
      * @access public
      */
-    function getUserInfo ()
+    function getUserInfo()
     {
         return $this->userinfo;
     }
@@ -711,7 +848,7 @@ class PEAR_Error
      * @return string debug information
      * @access public
      */
-    function getDebugInfo ()
+    function getDebugInfo()
     {
         return $this->getUserInfo();
     }
@@ -755,9 +892,6 @@ class PEAR_Error
                            $callback, $this->error_message_prefix,
                            $this->userinfo);
         }
-        if ($this->mode & PEAR_ERROR_CALLBACK) {
-            $modes[] = 'callback';
-        }
         if ($this->mode & PEAR_ERROR_PRINT) {
             $modes[] = 'print';
         }
